#ifndef WLR_CONFIG_H
#define WLR_CONFIG_H

#define WLR_HAS_DRM_BACKEND 1
#define WLR_HAS_LIBINPUT_BACKEND 1
#define WLR_HAS_X11_BACKEND 1

#define WLR_HAS_GLES2_RENDERER 1
#define WLR_HAS_VULKAN_RENDERER 1

#define WLR_HAS_GBM_ALLOCATOR 1

#define WLR_HAS_XWAYLAND 1

#endif
